#version 450

layout (local_size_x = 256) in;

layout (std140, binding = 0) uniform Camera {
    mat4 transformationProjectionMatrix;
    mat4 viewProjectionInverseMatrix;
    mat4 viewMatrix;
    mat4 projectionMatrix;
    ivec2 viewport;
    vec3 eyesPos;
} camera;

layout (push_constant) uniform Uniforms {
    mat4 modelMatrix;
    vec2 viewport;
    int count;
} uniforms;

layout (binding = 1) readonly buffer InputBuffer {
    vec4 position[];
} comp_in;

layout (binding = 2) buffer OutputBuffer {
    vec2 position[];
} comp_out;

void main() {
    uint gID = gl_GlobalInvocationID.x;

    if (gID < uniforms.count) {
        vec4 worldPos = uniforms.modelMatrix * comp_in.position[gID];
        vec4 clipSpace = camera.transformationProjectionMatrix * worldPos;
        vec3 ndcSpace = clipSpace.xyz / clipSpace.w;
        ndcSpace = vec3(ndcSpace.x, -ndcSpace.y, ndcSpace.z);
        vec2 result = ((ndcSpace.xy + 1.0) / 2.0) * uniforms.viewport;

        comp_out.position[gID] = vec2(result.x, uniforms.viewport.y - result.y);
    }
}
